Package: biobtreeR
Type: Package
Title: Using biobtree tool from R
Version: 1.25.0
Author: Tamer Gur
Maintainer: Tamer Gur <tgur@ebi.ac.uk>
Description: The biobtreeR package provides an interface to [biobtree](https://github.com/tamerh/biobtree) tool which covers large set of bioinformatics datasets and allows search and chain mappings functionalities.
License: MIT + file LICENSE
Encoding: UTF-8
LazyLoad: yes
NeedsCompilation: no
URL: https://github.com/tamerh/biobtreeR
BugReports: https://github.com/tamerh/biobtreeR/issues
VignetteBuilder: knitr
Suggests: BiocStyle, knitr,testthat,rmarkdown,markdown
Imports: httr, httpuv, stringi,jsonlite,methods,utils
biocViews: Annotation
RoxygenNote: 6.1.1
git_url: https://git.bioconductor.org/packages/biobtreeR
git_branch: devel
git_last_commit: c98ce19
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:29:54 UTC; biocbuild

importFrom(httr,HEAD,content,GET)
importFrom(httpuv,encodeURIComponent)
importFrom(jsonlite,fromJSON)
importFrom(stringi,stri_trim)
importFrom(methods, new)
importFrom(utils, download.file, untar, unzip)

export(bbUseOutDir,bbBuiltInDB,bbBuildCustomDB,bbStart,bbStop,bbSearch,bbMapping,bbEntry,bbEntryFilter,bbEntryPage,bbListAttrs,bbListDatasets)

## biobtreeR 

Bioconductor R package for genomic research via [biobtree](https://github.com/tamerh/biobtree). It aims to provide an alternative to existing packages with abilty process large and diverse datasets effectievly and allows executing basic or advanced queries .

For more detail and documentation check the Bioconductor [page](https://bioconductor.org/packages/3.11/bioc/html/biobtreeR.html)


## Install
```r

if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("biobtreeR")

```

## Usage

```r
  
  library(biobtreeR)
  
  # directory for the tool files 
  bbUseOutDir("specify your directory")
  
  # default database for most studied dataset and organism genomes 
  # once it is retrieved it is saved to your directory for later reuse
  # check document for included dataset or other builtin databases or build custom data
  bbBuiltInDB()

  # starts server for executing queries inside R pipelines and provide web ui for expolaration with examples
  # web interface address http://localhost:8888/ui/
  bbStart()

```

## Demo

Demo of web interface which covers all the datasets with examples

https://www.ebi.ac.uk/~tgur/biobtree/

#' @title Start biobtreeR
#'
#' @description Once target datasets is built with \code{bbBuildData} this function used to start biobtree server
#' in the background for performing search/mapping queries.
#'
#' @return character
#'
#' @examples
#'
#' bbStart()
#' bbStop()
#'
bbStart<-function() {

    if(isbbRunning()){
      print("There is a running biobtree which can be used or first bbStop() to start again")
      return(FALSE)
    }

    rootDir<-getwd()

    conf<-getConfig()

    if(length(conf@bbDir)==0){
      stop("out directory is not set. It must be set first with useOutDir function")
    }

    bbDir<-conf@bbDir

    tryCatch(
      {

        print(bbDir)

        setwd(bbDir)

        execFile <- bbExeFile()


        if(!file.exists(file.path(bbDir,"out","db","db.meta.json"))){
          stop("Built data couldn't found. Make sure data built or directory set correctly")
        }

        system2(execFile,args = "--no-web-popup web",wait = FALSE)

        # wait here until biobtree data process complete
        print("Starting biobtree...")
        elapsed<-0
        timeoutDuration<-50
        while(TRUE){

          if (elapsed > timeoutDuration){
            bbStop()
            setwd(rootDir)
            stop("biobtree could not started. Please check that you have built data correctly")
          }

          Sys.sleep(1)

          if(!isbbRunning()){
            elapsed<-elapsed+1
            next
          }

          setConfig()
          return("biobtreeR started")

        }


      },finally = {
        setwd(rootDir)
      }

    )


}

#'
#' @title Stop biobtree
#'
#' @description Stops running background biobtree process which started with \code{bbStart}
#'
#' @return returns empty
#'
#' @examples
#' bbStop()
bbStop <- function(){

    if (Sys.info()['sysname'] == "Windows") {

        system2("taskkill",args = "/IM biobtree.exe /F")

    }else if (Sys.info()['sysname'] == "Darwin"){

      system2("killall",args = "biobtree")

    }else if (Sys.info()['sysname'] == "Linux"){

      system2("killall",args = "biobtree")

    }

}

#
#Checks for running biobtree if found use it
#
isbbRunning <- function(biobtreeURL=NULL){

  conf<- getConfig()

  response <- tryCatch(
    HEAD(conf@metaEndpoint),
    error=function(e) e
  )

  if(is.null(response)){
    return(FALSE)
  }

  return(!inherits(response, "error"))

}

#' @title List available datasets
#'
#' @description Lists the available source and target datasets with their numeric identifiers.
#'
#' @return returns datasets
#'
#' @examples
#'
#' bbListDatasets()
#'
bbListDatasets <- function(){

  conf<-getConfig()

  if(length(conf@datasetIDs)>0){
    return(conf@datasetIDs)
  }

  if(!file.exists(file.path(conf@bbDir,"conf","source.dataset.json"))){
    bbBuildCustomDB(rawArgs="install")
  }else if (length(conf@datasetIDs)==0){
    setConfig()
  }

  return(getConfig()@datasetIDs)

}

#' @title Retrieve attributes of dataset
#'
#' @description Provides list of available attributes for a dataset to use in search and mapping queries.
#'
#' @param dataset Dataset identifier
#'
#' @return attributes names
#'
#'
#' @examples
#'
#' bbListAttrs("hgnc")
#' bbListAttrs("ensembl")
#'

bbListAttrs <- function(dataset){


  attrList<-function(){

    conf<-getConfig()

    if (!(dataset %in% names(conf@datasetMeta))){
      stop("Invalid dataset")
    }

    attrsStr<-conf@datasetMeta[[dataset]]$attrs

    if (!is.null(attrsStr)){
      return(unlist(strsplit(attrsStr,",")))
    }else{
      return(c())
    }
  }

  conf<-getConfig()

  if(length(conf@datasetMeta)>0){
    return(attrList())
  }

  if(!file.exists(file.path(conf@bbDir,"conf","source.dataset.json"))){
    bbBuildCustomDB(rawArgs="install")
  }else if (length(conf@datasetMeta)==0){
    setConfig()
  }

  return(attrList())

}

biobtreeREnv <- new.env(parent = emptyenv())
bbConfig<-NULL

#' @title Class for biobtreeR config
#'
#' @description This class holds the datasets meta data and web service endpoints and used while executing the search/mapping queries.
#' Instance of this class with name bbConfig is globally set by bbStart function. About dataset meta data, this class instance
#' holds all the datasets unqiue identifers, entry url templates etc. In biobtree each dataset has unique character
#' and numeric identifier. For instance Uniprot's charachter identifier is "uniprot" and numeric identifier is 1.
#' When performing queries the dataset charachter identifier is used for convinience but in actual database it is saved numerically.
#'
setClass("bbConfig", representation(bbDir="character",remote="logical",endpoint="character",metaEndpoint="character",datasetIDs="data.frame", datasetMeta="list", datasetMetaByNum="list" ))


#' @title Output directory for biobtreeR
#'
#' @description Allows to set the directory for the package for its files. It is required to set a valid directory.
#'
#' @param outDir path for the output directory.
#'
#' @return returns empty
#'
#' @examples
#'
#' bbUseOutDir(tempdir())
#'
bbUseOutDir<-function(outDir){

  if(!file.exists(file.path(outDir))){
    stop("Specified outDir is not exist")
  }

  conf<-getConfig()
  conf@bbDir=outDir
  assign("bbConfig",conf, envir = biobtreeREnv)
  return(0)

}

setOutDir<-function(outDir){

  bbUseOutDir(outDir)

}



getConfig <- function(){

  if (exists("bbConfig", envir=biobtreeREnv)){
    return(get("bbConfig", envir=biobtreeREnv))
  }

  localEndpoint="http://localhost:8888"

  conf<-new("bbConfig",
            endpoint=localEndpoint,
            metaEndpoint=paste0(localEndpoint,"/ws/meta"),
            remote=FALSE)

  assign("bbConfig",conf, envir = biobtreeREnv)

  bbExeFile()

  return(conf)

}

setConfig<-function(){

  conf<-getConfig()

  datasetIDs <- list()
  datasetMeta <- list()
  datasetMetaByNum <- list()

  conf@endpoint="http://localhost:8888"
  conf@metaEndpoint=paste0(conf@endpoint,"/ws/meta")

  srcFile<-file.path(paste0(conf@bbDir,"/conf/","source.dataset.json"))
  srcString<-readChar(srcFile,file.info(srcFile)$size)
  srcList <- fromJSON(srcString,simplifyVector = FALSE,encoding = "UTF-8")

  defFile<-file.path(paste0(conf@bbDir,"/conf/","default.dataset.json"))
  defString<-readChar(defFile,file.info(defFile)$size)
  defList <- fromJSON(defString,simplifyVector = FALSE,encoding = "UTF-8")

  optFile<-file.path(paste0(conf@bbDir,"/conf/","optional.dataset.json"))
  optString<-readChar(optFile,file.info(optFile)$size)
  optList <- fromJSON(optString,simplifyVector = FALSE,encoding = "UTF-8")

  datasetMeta <- appendList(appendList(srcList,defList),optList)

  for (v in names(datasetMeta)){
    idnum<-datasetMeta[[v]]$id
    datasetMetaByNum[idnum] <- datasetMeta[v]
    datasetMetaByNum[[idnum]]$id <- v
  }

  datasetNumericIds <- sort(as.numeric(names(datasetMetaByNum)))


  if(length(datasetMeta)==0 || length(datasetMetaByNum)==0){
    stop("Error while retrieving datasets information.")
  }

  strID<-vector(length =length(datasetNumericIds) )
  numID<-c()
  types<-c()
  i=1
  for(v in datasetNumericIds){
    strID[[i]]<-datasetMetaByNum[[paste(v)]]$id
    numID[[i]]<-paste(v)
    if(i<30){
      types[i]<-"data source&target"
    }else{
      types[i]<-"target"
    }
    i=i+1
  }

  datasetIDs<-data.frame(id=strID,numeric_id=numID,type=types)

  conf@datasetIDs=datasetIDs
  conf@datasetMeta=datasetMeta
  conf@datasetMetaByNum=datasetMetaByNum
  assign("bbConfig",conf, envir = biobtreeREnv)

}

appendList <- function (x, val)
{
  stopifnot(is.list(x), is.list(val))
  xnames <- names(x)
  for (v in names(val)) {
    x[[v]] <- if (v %in% xnames && is.list(x[[v]]) && is.list(val[[v]]))
      appendList(x[[v]], val[[v]])
    else c(x[[v]], val[[v]])
  }
  x
}

bbExeFile<- function(){

  conf<-getConfig()

  if (Sys.info()['sysname'] == "Windows") {

    if(!file.exists("biobtree.exe")){

      latestUrl<-paste0("https://github.com/tamerh/biobtree/releases/download/",latestbbVersion(),"/biobtree_Windows_64bit.zip")
      download.file(latestUrl,"biobtree_Windows_64bit.zip",mode="wb")
      unzip("biobtree_Windows_64bit.zip")
      file.remove("biobtree_Windows_64bit.zip")

    }
    bbExe<-file.path(conf@bbDir,"biobtree.exe")
    return(bbExe)

  }else if (Sys.info()['sysname'] == "Darwin"){

    if(!file.exists("biobtree")){

      latestUrl<-paste0("https://github.com/tamerh/biobtree/releases/download/",latestbbVersion(),"/biobtree_MacOS_64bit.tar.gz")
      download.file(latestUrl,"biobtree_MacOS_64bit.tar.gz",mode="wb")
      system2("tar",args=" -xzvf biobtree_MacOS_64bit.tar.gz")
      file.remove("biobtree_MacOS_64bit.tar.gz")

    }
    bbExe<-file.path(conf@bbDir,"biobtree")
    return(bbExe)

  }else if (Sys.info()['sysname'] == "Linux"){

    if(!file.exists("biobtree")){

      latestUrl<-paste0("https://github.com/tamerh/biobtree/releases/download/",latestbbVersion(),"/biobtree_Linux_64bit.tar.gz")
      download.file(latestUrl,"biobtree_Linux_64bit.tar.gz",mode="wb")
      system2("tar",args=" -xzvf biobtree_Linux_64bit.tar.gz")
      file.remove("biobtree_Linux_64bit.tar.gz")

    }
    bbExe<-file.path(conf@bbDir,"biobtree")
    return(bbExe)

  }

  stop("Unsupported OS -> ",Sys.info()['sysname'])

}

latestbbVersion<- function(){

  relurl<-GET("https://github.com/tamerh/biobtree/releases/latest")
  if (relurl$status_code!=200){
    stop("Error while connecting github for retrieving file")
  }
  splittedurl<-unlist(strsplit(relurl$url,"/"))
  return(splittedurl[length(splittedurl)])

}

latestbbConfVersion<- function(){

  relurl<-GET("https://github.com/tamerh/biobtree-conf/releases/latest")
  if (relurl$status_code!=200){
    stop("Error while connecting github for retrieving file")
  }
  splittedurl<-unlist(strsplit(relurl$url,"/"))
  return(splittedurl[length(splittedurl)])

}

#' @title Retrieve entry
#'
#' @description Returns entry for an identifier and dataset. Entry contains all the data raw data for and entry such as mappings, attiributes and paging info if exists.
#'
#' @param identifer Identifer for the entry. Note that keywords are not accepted. For instance insted of "vav_human" keyword "p15498" identifier must be passed
#' @param source Dataset identifier
#'
#' @return returns biobtree json object
#'
#' @author Tamer Gur
#'
#' @examples
#'
#' bbStart() # if not already started
#' bbEntry("HGNC:12009","hgnc")
#'

bbEntry <- function(identifer,source){

  searchurl <- paste0(getConfig()@endpoint,"/ws/entry/?i=",encodeURIComponent(identifer),"&s=",source)
  res <- fromJSON(searchurl,simplifyVector = FALSE,encoding = "UTF-8")
  return(res)
}

#' @title Retrieve entry with filtered dataset
#'
#' @description Similar with entry retrieval but filtered mapping entries with given datasets.
#'
#'
#' @param identifer Identifer for the entry.
#' @param source Dataset identifier
#' @param filters Comma seperated dataset identifer to retrieve
#' @param page Page index if results is more than default biobtree paging size.
#'
#' @return returns biobtree json object
#'
#' @author Tamer Gur
#'
#' @examples
#'
#'
#' bbStart() # if not already started
#' bbEntryFilter("HGNC:12009","hgnc","uniprot,ensembl")
#'
#'


bbEntryFilter <-function(identifer,source,filters,page=NULL) {

  searchurl = paste0(getConfig()@endpoint,"/ws/filter/?i=",encodeURIComponent(identifer),'&s=', source , '&f=' ,filters)

  if (length(page) > 0) {
    searchurl =paste0(searchurl,"&p=" , page)
  }

  res <- fromJSON(searchurl,simplifyVector = FALSE,encoding = "UTF-8")

  return(res)

}

#' @title Retrieve entry result page
#'
#' @description If an entry contains large set of mapping entries it is paginated by biobtree with confiGured paging size.
#' This function retrieve these paging for an entry. Biobtree paging size for each entry is 200.
#'
#' @param identifer Identifer for the entry.
#' @param source Dataset identifier
#' @param page Page index it starts from 0
#' @param totalPage Total number of page for the entry. This value needs to calculate by user via using total number of entries which is available at the root result for the entry
#' and divide it to the paging sizeb of 200
#'
#' @return returns biobtree json object
#'
#' @author Tamer Gur
#'
#' @examples
#'
#' bbStart() # if not already started
#' bbEntryPage("ENSG00000141956","ensembl",0,0)
#'
#'

bbEntryPage <- function (identifer, source, page, totalPage) {

  searchurl = paste0(getConfig()@endpoint,"/ws/page/?i=" ,identifer , '&s=' , source , '&p=' , page , '&t=' , totalPage)

  res <- fromJSON(searchurl,simplifyVector = FALSE,encoding = "UTF-8")

  return(res)

}

#' @title Chain mapping and filtering
#'
#' @description Chain mapping identifiers or keywords with filtering and retrieving attributes if available.
#'
#' @param terms Input terms for the mapping. Same with search functionality they can be comma seperated identifers or keywords
#' @param mapfilter Mapping query which consist of map and optional filter functions in the form of map(dataset).filter(Boolean query expression)
#' The boolean expressions are based on datasets attributes and dataset attributes can be list with bbAttr function.
#' Dataset attributes which used in the filters starts with their dataset name. In biobtree boolean expressions
#' feature has been implemented via Google common expression language so its full capability can be checked in its documentation.
#' @param source Optional dataset identifiers for searching input terms within the given dataset.
#' @param page Optional parameter works similar with bbSearch page paramter.
#' @param lite By default it is TRUE and allow function return quickly with data.frame with mapping identifiers and attributes. If set to TRUE function return raw results converted from json.
#' @param limit Limits the number of mapping results. By default without any limit all the results returned.
#' @param inattrs Optional comma seperated attribute names for input identifiers and if available their values includes in result data.frame
#' @param attrs Optional comma seperated attribute names for mapping identifiers and if available their values includes in result data.frame
#' @param showInputColumn Optional logical parameter to show the input identifers in the result data.frame
#'
#' @return returns mapping results in data.frame by default if lite set it true returns json object
#'
#' @author Tamer Gur
#'
#' @examples
#' bbStart()
#'
#' bbMapping("tpi1",'map(uniprot)')
#'
#' bbMapping("shh",'map(ensembl)')
#'
#' \dontrun{
#' # run these examples with building the default dataset with bbBuildData()
#' #Map protein to its go terms and retrieve go term types
#' bbMapping("AT5G3_HUMAN",'map(go)',attrs = "type")
#'
#' #Map protein to its go terms with filter by its type and retrieve their types
#' bbMapping("AT5G3_HUMAN",'map(go).filter(go.type=="biological_process")',attrs = "type")
#'
#' #Map gene names to exon identifiers and retrieve the region
#' bbMapping("ATP5MC3,TP53",'map(transcript).map(exon)',attrs = "seq_region_name")
#'
#' #Map Affymetrix identifiers to Ensembl identifiers and gene names
#' bbMapping("202763_at,213596_at,209310_s_at",source ="affy_hg_u133_plus_2"
#' ,'map(transcript).map(ensembl)',attrs = "name")
#'
#'}
#'
bbMapping <- function(terms, mapfilter, page=NULL, source=NULL,lite=TRUE,limit=1000,inattrs=NULL,attrs=NULL,showInputColumn=FALSE){

  wsurl <- function(terms,mapfilter,source,page){

    mfurl <- paste0(getConfig()@endpoint,"/ws/map/?i=",encodeURIComponent(terms),"&m=",encodeURIComponent(mapfilter))

    if (length(page) > 0) {
      mfurl <-paste0(mfurl,"&p=" , page)
    }
    if (length(source) > 0) {
      mfurl <-paste0(mfurl,"&s=" ,source)
    }

    return(mfurl)

  }

  totalMapping=0
  urlstr <- wsurl(terms,mapfilter,source,page)
  res <- fromJSON(urlstr,simplifyVector = FALSE,encoding = "UTF-8")
  if (length(res$Err)>0){
      return(res)
  }

  results <- res$results

  for (r in results){
    totalMapping=totalMapping+length(r$targets)
  }

  lastpagekey<-res$nextpage

  if(length(lastpagekey)>0 && totalMapping< limit ){

    while(length(lastpagekey)>0 && totalMapping<limit){

      urlstr<- wsurl(terms,mapfilter,source,lastpagekey)
      newres <- fromJSON(urlstr,simplifyVector = FALSE,encoding = "UTF-8")

      if (length(newres$Err)>0){
        return(res$Err)
      }

      for(i in seq_len(length(newres$results))){
          for(j in seq_len(length(results))){
            if ( identical(newres$results[[i]]$source$doman_id,results[[j]]$source$doman_id) && identical(newres$results[[i]]$source$identifier,results[[j]]$source$identifier)) {
              results[[j]]$targets<-append(results[[j]]$targets,newres$results[[i]]$targets)
              totalMapping=totalMapping+length(newres$results[[i]]$targets)
              break
            }
          }
      }

      lastpagekey=newres$nextpage

    }
  }

  if (lite){

    multiInput<-FALSE
    if(length(results)>1 || showInputColumn){
      multiInput<-TRUE
    }

    inAttrsNames<-NULL
    inAttrsVals<-NULL

    if(length(inattrs)>0){
      multiInput<-TRUE
      inAttrsNames<-unlist(strsplit(inattrs,","))
      inAttrsVals<-list()
      for(at in inAttrsNames){
        inAttrsVals[at] <- c()
      }
    }

    attrsNames<-NULL
    attrsVals<-NULL

    if(length(attrs)>0){
      attrsNames<-unlist(strsplit(attrs,","))
      attrsVals<-list()
      for(at in attrsNames){
        attrsVals[at] <- c()
      }
    }

    if(multiInput){
      in_id<-c()
      in_source<-c()
    }
    map_id<-c()

    index=1
    for(i in seq_len(length(results))){

      for(j in seq_len(length(results[[i]]$targets))){

        map_id[index]<-results[[i]]$targets[[j]]$identifier

        if(j==1 && multiInput){
            if(length(results[[i]]$source$keyword)>0){
              in_id[index]<-paste0(results[[i]]$source$keyword,"-",results[[i]]$source$identifier)
            }else{
              in_id[index]<-results[[i]]$source$identifier
            }
            source_id<-results[[i]]$source$dataset
            in_source[index]<-getConfig()@datasetMetaByNum[[paste(source_id)]]$id
        }else if(multiInput){
            in_id[index]<-"-"
            in_source[index]<-"-"
        }

        if(length(attrsNames)>0){
          attrName<-names(results[[i]]$targets[[j]]$Attributes)[1]
          if(attrName!="Empty"){
            attrsPath<-paste0("results[[i]]$targets[[j]]$Attributes$",attrName)
            for(at in attrsNames){
              atPath<-paste0(attrsPath,"$",at)
              
              atVal<-eval(parse(text=atPath))
              if (is.null(atVal)) {
                attrsVals[[at]][index]<-""
              }else{
                attrsVals[[at]][index]<-eval(parse(text=atPath))  
              }
              
            }
          }else{
            for(at in attrsNames){
              attrsVals[[at]][index]<-""
            }
          }

        }

        if(length(inAttrsNames)>0){
          attrsPath<-paste0("results[[i]]$source$Attributes$",names(results[[i]]$source$Attributes)[1])
          for(at in inAttrsNames){
            atPath<-paste0(attrsPath,"$",at)
            atVal<-eval(parse(text=atPath))
          
            if (is.null(atVal)) {
              inAttrsVals[[at]][index]<-""
            }else{
              inAttrsVals[[at]][index]<-eval(parse(text=atPath))  
            }
          
          }
        }

        index=index+1
      }

    }

    if(multiInput){
      df<-data.frame(input=in_id,input_dataset=in_source)

      if(length(inAttrsNames)>0){
        for(at in inAttrsNames){
          df[at]<-inAttrsVals[at]
        }
      }

      df["mapping_id"]=map_id

    }else{
      df<-data.frame(mapping_id=map_id)
    }

    if(length(attrsNames)>0){
      for(at in attrsNames){
        df[at]<-attrsVals[at]
      }
    }

    return(df)

  }

  if(length(lastpagekey)>0){
    return(list(results=results,nextpage=lastpagekey))
  }

  return(list(results=results))

}

#' @title Search identifiers or special keywords
#'
#' @description Search identifiers or special keywords terms uniformly and resolve their actual unique identifiers and datasets. Keywords
#' can be several things for instance for uniprot an accession like "vav_human" can be a keyword which points to its original
#' identifier "P15498". Or gene name can be also a keyword like "tpi1" which could points multiple dataset like ensembl and hgnc.
#'
#' @param terms Comma seperated identifers or keywords
#' @param source Optional dataset identifiers to search only within this dataset.
#' @param filter Filter expression useful to filter out results when a keyword point several results. For instance if the biobtree index with multiple organism
#' a same gene search could hit several results for different species to filter only a specific species a filter can apply to search function.
#' @param page By default no need to pass this parameter since it returns all the results. It can be used with limit parameter for very large results to process them in paginated manner.
#' About paging every long search or mapping result paginated in biobtree and for paginated results every response contains a key to get the next page results. So if this
#' parameter is set with this key specified next page results returned for the given search term.
#' @param showURL allows returning the dataset source url
#' @param lite By default it is TRUE and allow function return quickly with data.frame containing most important fields. If set to TRUE function return raw results converted from json.
#' @param limit  Limits the number of search results. By default without any limit all the results returned.
#'
#' @return returns search results in data.frame by default if lite set it true returns json object
#'
#' @author Tamer Gur
#'
#' @examples
#'
#' bbSearch("hunk,vav_human")
#'
#' bbSearch("hunk","ensembl",filter='ensembl.genome=="homo_sapiens"')
#'
#'

bbSearch <- function(terms,source=NULL,filter=NULL, page=NULL,limit=1000,showURL=FALSE,lite=TRUE){

  wsurl <- function(terms,source,filter,page,lite,showURL){

      searchurl <- paste0(getConfig()@endpoint,"/ws/?i=",encodeURIComponent(terms))

      if (length(page) > 0) {
        searchurl <-paste0(searchurl,"&p=" , page)
      }
      if (length(filter) > 0) {
        searchurl <- paste0(searchurl,"&f=" , filter)
      }
      if (length(source) > 0) {
        searchurl <-paste0(searchurl,"&s=" ,source)
      }
      if (!lite) {
        searchurl <-paste0(searchurl,"&d=d")
      }
      if (showURL){
        searchurl <- paste0(searchurl,"&u=u")
      }
      return(searchurl)
  }


  urlstr <- wsurl(terms,source,filter,page,lite,showURL)

  res <- fromJSON(urlstr,simplifyVector = FALSE,encoding = "UTF-8")

  if (length(res$Err)>0){
    return((res))
  }

  results<-res$results

  if(length(res$nextpage)>0 && length(results)< limit ){
        lastpagekey=res$nextpage
        while(length(lastpagekey)>0 && length(results)<limit){
          urlstr<- wsurl(terms,source,filter,lastpagekey,lite,showURL)
          r <- fromJSON(urlstr,simplifyVector = FALSE,encoding = "UTF-8")
          if (length(r$Err)>0){
            return(r)
          }
          results<-append(results,r$results)
          lastpagekey<-r$nextpage
        }
  }


  if (lite){
    input<-c()
    id<-c()
    source<-c()
    mappingCount<-c()
    if(showURL){
      urls<-c()
    }
    i=1
    for(r in results){
      if (length(r$keyword)>0){
        input[i]<-r$keyword
      }else{
        input[i]<-r$identifier
      }
      id[i]<-r$identifier
      mappingCount[i]<-r$count
      source[i]<-getConfig()@datasetMetaByNum[[paste(r$dataset)]]$id
      if(showURL){
        urls[i]<-r$url
      }
      i=i+1
    }

    df<-data.frame(input=input,identifier=id,dataset=source,mapping_count=mappingCount)
    if(showURL){
      df["url"]<-urls
    }
    return(df)
  }
  return(res)
}

#' @title Get pre build biobtree database
#'
#' @description Pre build biobtree database for commonly studied datasets and model organism genomes. Once this function called it retrieves
#' the pre build database saves to users output directory.
#'
#' @param type built in database type accepted values are 1,2,3 and 4. Currently there are 4 different builtin database;
#' Type 1
#' Requires ~ 5 GB free storage
#' Included datasets hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro
#' Included uniprot proteins and ensembl genomes belongs to following organisms
#'
#' homo_sapiens 9606 --> ensembl
#' danio_rerio 7955 zebrafish --> ensembl
#' gallus_gallus 9031 chicken --> ensembl
#' mus_musculus 10090 --> ensembl
#' Rattus norvegicus 10116 ---> ensembl
#' saccharomyces_cerevisiae 4932--> ensembl,ensembl_fungi
#' arabidopsis_thaliana 3702--> ensembl_plants
#' drosophila_melanogaster 7227 --> ensembl,ensembl_metazoa
#' caenorhabditis_elegans 6239 --> ensembl,ensembl_metazoa
#' Escherichia coli 562 --> ensembl_bacteria
#' Escherichia coli str. K-12 substr. MG1655 511145 --> ensembl_bacteria
#' Escherichia coli K-12 83333 --> ensembl_bacteria
#'
#' Type 2
#' Requires ~ 5 GB free storage
#' Instead of genomes in in the type 1 it contains human and all the mouse strains genomes with their uniprot proteins.
#' In addition hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro datasets are included
#'
#' Type 3
#' Requires ~ 4 GB storage
#' Contains no genome but it contains all the uniprot data with hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro
#'
#' Type 4
#' Requires ~ 13 GB storage
#' Contains no genome but full uniprot and chembl data with hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro
#'
#' @return returns empty
#'
#' @author Tamer Gur
#'
#' @examples
#'
#' bbUseOutDir(tempdir()) # temp dir for demo purpose
#' bbBuiltInDB("demo") # small demo database for real database use 1, 2, 3 or 4
#'
#'
bbBuiltInDB<- function(type="1"){

  if(isbbRunning()){
    stop("There is a running biobtree stop with bbStop()")
  }

  rootDir<-getwd()

  conf<-getConfig()

  if(length(conf@bbDir)==0){
    stop("out directory is not set")
  }

    tryCatch(
    {

      setwd(conf@bbDir)


      execFile <- bbExeFile()

      args<-paste0(" --pre-built ",type," install")

      system2(execFile,args=args)

    },finally = {

      setwd(rootDir)

    }

  )

}


#' @title Build custom DB
#'
#' @description biobtree covers all the genomes in ensembl and ensembl genomes. If the the studied organism genome is not included
#' in the default pre built in databases then this function is used and build the biobtree database locally for given genomes.
#'
#' @param taxonomyIDs Comma seperated list of taxonomy identifiers for building the genomes
#'
#' @param rawArgs For using all available biobtree command line arguments directly
#'
#' @return returns empty
#'
#' @author Tamer Gur
#'
#' @examples
#'
#' \dontrun{
#'
#' bbUseOutDir("your directory path")
#' bbBuildCustomDB(taxonomyIDs="1408103,206403")
#'
#'}
#'
bbBuildCustomDB<- function(taxonomyIDs=NULL,rawArgs=NULL){

  if(isbbRunning()){
    stop("There is a running biobtree stop with bbStop()")
  }

  rootDir<-getwd()

  conf<-getConfig()

  if(length(conf@bbDir)==0){
    stop("out directory is not set")
  }

  if(file.exists(file.path(conf@bbDir,"out","db","db.meta.json"))){
    stop("Previously built data found in current out directory. Either delete the out directory inside current out directory or use new directory")
  }

  bbDir<-conf@bbDir

  tryCatch(
    {

      setwd(bbDir)

      execFile <- bbExeFile()

      if(length(rawArgs)>0){

        system2(execFile,args=rawArgs)

      }else{

        if(length(taxonomyIDs)==0){
          stop("At least one taxonomy identifier is required to build custom data")
        }

        # to speed up the process get raw index data of builtin db 3
        bbBuiltInDB("3r")

        args<-paste0(" -tax ",taxonomyIDs," --keep --ensembl-orthologs"," build")

        system2(execFile,args=args)

      }

      setConfig()

    },finally = {

      setwd(rootDir)

    }

  )

}

---
title: "The biobtreeR users guide"
author: "Tamer Gür"
graphics: no
package: biobtreeR
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{The biobtreeR users guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


# Introduction

The biobtreeR package provides an interface to [biobtree](https://github.com/tamerh/biobtree) tool which allows mapping the bioinformatics datasets via identifiers and special keywors with simple or advance chain query capability.

# Getting started 

```{r}
library(biobtreeR)

# Create an folder and set as an output directory 
# It is used for database and configuration files

# temporary directory is used for demonstration purpose
bbUseOutDir(tempdir())

```


## Build database 
For mapping queries, biobtreeR use a database which stored in local storage. Database can be built 2 ways, first way is to retrieve pre built database. These database consist of commonly studied datasets and model organism and updated regularly following the major uniprot and ensembl data releases. 


```{r}
# Called once and saves the built in database to local disk

# Included datasets hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro
# Included uniprot proteins and ensembl genomes belongs to following organisms:
# homo_sapiens, danio_rerio(zebrafish), gallus_gallus(chicken), mus_musculus, Rattus norvegicus, saccharomyces_cerevisiae, 
# arabidopsis_thaliana, drosophila_melanogaster, caenorhabditis_elegans, Escherichia coli, Escherichia coli str. K-12 substr. MG1655, Escherichia coli K-12

# Requires ~ 6 GB free storage
bbBuiltInDB()

```

For the genomes which are not included in the pre built database can be built in local computer. All the ensembl and ensembl genomes organisms are supported.
List of these genomes and their taxonomy identifiers can be seen from ensembl websites [1](https://www.ensembl.org/info/about/species.html),[2](http://bacteria.ensembl.org/species.html),[3](http://fungi.ensembl.org/species.html),[4](http://plants.ensembl.org/species.html),[5](http://protists.ensembl.org/index.html),[6](http://metazoa.ensembl.org/species.html),
```{r eval=FALSE}
  
# multiple species genomes supported with comma seperated taxonomy identifiers
bbBuildCustomDB(taxonomyIDs = "1408103,206403")

```


## Start web server

Once database is retrieved or built to local disk queries are performed via lightweight local server. Local server provide web interface for data expoloration in addition to the R functions for performing queries for R pipelines. Local server runs as a background process so both web interface and R functions can be used at the same time once it is started. While web server running web interface can be accessed via address http://localhost:8888/ui


```{r}
 bbStart()
```


## Search identifiers and keywords

Searching dataset identfiers and keywords such as gene name or accessions is performed with bbSearch function by passing comma seperated terms.

```{r}
bbSearch("tpi1,vav_human,ENST00000297261")
```

If source parameter is passed search performed within the dataset.

```{r}
bbSearch("tpi1,ENSG00000164690","ensembl")
```

Search results url is retrieved with 

```{r}
bbSearch("tpi1,vav_human,ENST00000297261",showURL =TRUE)
```

## Chain mapping and filtering 
Mapping and filtering queries are performed via bibobtree's mapping query syntax which allowed chain mapping and filtering capability from identifiers to target identifiers or attributes. Mapping query syntax consist of single or multiple mapping queries in the format `map(dataset_id).filter(Boolean query expression).map(...).filter(...)...` and allow performing chain mapping among datasets. For mapping queries bbMapping function is used, for instance in following example, maps protein to its go terms and in the second query mapping has been done with filter.

```{r}
bbMapping("AT5G3_HUMAN",'map(go)',attrs = "type")
```

```{r}
bbMapping("AT5G3_HUMAN",'map(go).filter(go.type=="biological_process")',attrs = "type")
```

In the example for the first parameter single protein accession has been used but similar with bbSearch functions multiple identifers or keywords can be used. In the last query type attribute was used to filter mapping only with biological process go terms. Dataset attributes are used in the filters starts with their dataset name as in the above example it starts with `go.`

In order use in filter expressions, each datasets attributes lists with `bbListAttrs`function via sample identifier. For instance following query shows gene ontology attributes.

```{r}
bbListAttrs("go")
```

```{r}
bbListAttrs("uniprot")
```

# Example Use cases

In this section biobtreeR functionalities will be discussed in detail via gene and protein centric example use cases. For live demo of web interface including these use cases with additional chemistry centric use cases can be accessed via https://www.ebi.ac.uk/~tgur/biobtree/

## Gene centric use cases

Ensembl, Ensembl Genomes and HGNC datasets are used for gene related data. One of the most common gene related dataset identfiers are `ensembl`,`hgnc`,`transcript`,`exon`. Let's start with listing their attiributes,

```{r}
bbListAttrs("hgnc")
```

```{r}
bbListAttrs("ensembl")
```

```{r}
bbListAttrs("transcript")
```

```{r}
bbListAttrs("exon")
```

```{r}
bbListAttrs("cds")
```
Note that there are several other gene related datasets without attributes and can be used in mapping queries such as probesets, genebank and entrez etc. Full dataset list can be discovered with `bbListDatasets`. Now lets build example mapping queries,

**Map gene names to Ensembl transcript identifiers**

```{r}
res<-bbMapping("ATP5MC3,TP53",'map(transcript)')
head(res)
```

**Map gene names to exon identifiers and retrieve the region**

```{r}
res<-bbMapping("ATP5MC3,TP53",'map(transcript).map(exon)',attrs = "seq_region")
head(res)
```

**Map human gene to its ortholog identifiers**

```{r}
res<-bbMapping("shh",'filter(ensembl.genome=="homo_sapiens").map(ortholog)')
head(res)
```

**Map gene to its paralogs**

```{r}
bbMapping("fry,mog",'map(paralog)',showInputColumn = TRUE)
```


**Map ensembl identifier or gene name to the entrez identifier**
```{r}
bbMapping("ENSG00000073910,shh" ,'map(entrez)')
```

**Map refseq identifiers to hgnc identifiers**

```{r}
bbMapping("NM_005359,NM_000546",'map(hgnc)',attrs = "symbols")
```

**Get all Ensembl human identifiers and gene names on chromosome Y with lncRNA type**
```{r}
res<-bbMapping("homo_sapiens",'map(ensembl).filter(ensembl.seq_region=="Y" && ensembl.biotype=="lncRNA")',attrs = 'name')
head(res)
```

**Get CDS from genes**

```{r}
bbMapping("tpi1,shh",'map(transcript).map(cds)')
```

**Get all Ensembl human identifiers and gene names within or overlapping range**
```{r}
bbMapping("9606",'map(ensembl).filter((114129278>ensembl.start && 114129278<ensembl.end) || (114129328>ensembl.start && 114129328<ensembl.end))',attrs = "name")
```

In the above example as a first parameter taxonomy identifier is used instead of specifying as homo sapiens like in the previous example. Both of these usage are equivalent and produce same output as homo sapiens refer to taxonomy identifer 9606.

**Built in function for genomic range queries**

To simplfy previous use case query 3 builtin range query functions are provided. These functions are `overlaps`, `within` and `covers`. These functions can be used for `ensembl`, `transcript`, `exon` and `cds`
entries which have start and end genome coordinates. For instance previous query can be written following way with `overlaps` function which list all the overlapping genes in human with given range.

```{r}
bbMapping("9606",'map(ensembl).filter(ensembl.overlaps(114129278,114129328))',attrs = "name")
```



**Map Affymetrix identifiers to Ensembl identifiers and gene names**

```{r}
bbMapping("202763_at,213596_at,209310_s_at",source ="affy_hg_u133_plus_2" ,'map(transcript).map(ensembl)',attrs = "name")
```

Note that all mappings can be done with opposite way, for instance from gene name to Affymetrix identifiers mapping is performed following way

```{r}
bbMapping("CASP3,CASP4",'map(transcript).map(affy_hg_u133_plus_2)')
```

**Retrieve all the human gene names which contains TTY**
```{r}
res<-bbMapping("homo sapiens",'map(ensembl).filter(ensembl.name.contains("TTY"))',attrs = "name")
head(res)
```

## Protein centric use cases

Uniprot is used for protein related dataset such as protein identifiers, accession, sequence, features, variants, and mapping information to other datasets. Let's list some protein related datasets attributes and then execute example queries similary with gene centric examples,

```{r}
bbListAttrs("uniprot")
```

```{r}
bbListAttrs("ufeature")
```

```{r}
bbListAttrs("pdb")
```

```{r}
bbListAttrs("interpro")
```


**Map gene names to reviewed uniprot identifiers**

```{r}
bbMapping("msh6,stk11,bmpr1a,smad4,brca2","map(uniprot)",source ="hgnc")
```

**Filter proteins by sequence mass and retrieve protein sequences**

```{r}
bbMapping("clock_human,shh_human,aicda_human,at5g3_human,p53_human","filter(uniprot.sequence.mass > 45000)" ,attrs = "sequence$mass,sequence$seq")
```

**Helix type feature locations of a protein**

```{r}
bbMapping("shh_human",'map(ufeature).filter(ufeature.type=="helix")' ,attrs = "location$begin,location$end")
```

**Get all variation identifiers from a gene with given condition**

```{r}
bbMapping("tp53",'map(uniprot).map(ufeature).filter(ufeature.original=="I" && ufeature.variation=="S").map(variantid)',source = "hgnc")
```


# Stop web server
When working with biobtreeR completed, the biobtreeR web server should stop.

```{r}
bbStop()
```

# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search.R
\name{bbSearch}
\alias{bbSearch}
\title{Search identifiers or special keywords}
\usage{
bbSearch(terms, source = NULL, filter = NULL, page = NULL,
  limit = 1000, showURL = FALSE, lite = TRUE)
}
\arguments{
\item{terms}{Comma seperated identifers or keywords}

\item{source}{Optional dataset identifiers to search only within this dataset.}

\item{filter}{Filter expression useful to filter out results when a keyword point several results. For instance if the biobtree index with multiple organism
a same gene search could hit several results for different species to filter only a specific species a filter can apply to search function.}

\item{page}{By default no need to pass this parameter since it returns all the results. It can be used with limit parameter for very large results to process them in paginated manner.
About paging every long search or mapping result paginated in biobtree and for paginated results every response contains a key to get the next page results. So if this
parameter is set with this key specified next page results returned for the given search term.}

\item{limit}{Limits the number of search results. By default without any limit all the results returned.}

\item{showURL}{allows returning the dataset source url}

\item{lite}{By default it is TRUE and allow function return quickly with data.frame containing most important fields. If set to TRUE function return raw results converted from json.}
}
\value{
returns search results in data.frame by default if lite set it true returns json object
}
\description{
Search identifiers or special keywords terms uniformly and resolve their actual unique identifiers and datasets. Keywords
can be several things for instance for uniprot an accession like "vav_human" can be a keyword which points to its original
identifier "P15498". Or gene name can be also a keyword like "tpi1" which could points multiple dataset like ensembl and hgnc.
}
\examples{

bbSearch("hunk,vav_human")

bbSearch("hunk","ensembl",filter='ensembl.genome=="homo_sapiens"')


}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server.R
\name{bbStart}
\alias{bbStart}
\title{Start biobtreeR}
\usage{
bbStart()
}
\value{
character
}
\description{
Once target datasets is built with \code{bbBuildData} this function used to start biobtree server
in the background for performing search/mapping queries.
}
\examples{

bbStart()
bbStop()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lists.R
\name{bbListAttrs}
\alias{bbListAttrs}
\title{Retrieve attributes of dataset}
\usage{
bbListAttrs(dataset)
}
\arguments{
\item{dataset}{Dataset identifier}
}
\value{
attributes names
}
\description{
Provides list of available attributes for a dataset to use in search and mapping queries.
}
\examples{

bbListAttrs("hgnc")
bbListAttrs("ensembl")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entry.R
\name{bbEntryPage}
\alias{bbEntryPage}
\title{Retrieve entry result page}
\usage{
bbEntryPage(identifer, source, page, totalPage)
}
\arguments{
\item{identifer}{Identifer for the entry.}

\item{source}{Dataset identifier}

\item{page}{Page index it starts from 0}

\item{totalPage}{Total number of page for the entry. This value needs to calculate by user via using total number of entries which is available at the root result for the entry
and divide it to the paging sizeb of 200}
}
\value{
returns biobtree json object
}
\description{
If an entry contains large set of mapping entries it is paginated by biobtree with confiGured paging size.
This function retrieve these paging for an entry. Biobtree paging size for each entry is 200.
}
\examples{

bbStart() # if not already started
bbEntryPage("ENSG00000141956","ensembl",0,0)


}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{bbUseOutDir}
\alias{bbUseOutDir}
\title{Output directory for biobtreeR}
\usage{
bbUseOutDir(outDir)
}
\arguments{
\item{outDir}{path for the output directory.}
}
\value{
returns empty
}
\description{
Allows to set the directory for the package for its files. It is required to set a valid directory.
}
\examples{

bbUseOutDir(tempdir())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lists.R
\name{bbListDatasets}
\alias{bbListDatasets}
\title{List available datasets}
\usage{
bbListDatasets()
}
\value{
returns datasets
}
\description{
Lists the available source and target datasets with their numeric identifiers.
}
\examples{

bbListDatasets()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildData.R
\name{bbBuiltInDB}
\alias{bbBuiltInDB}
\title{Get pre build biobtree database}
\usage{
bbBuiltInDB(type = "1")
}
\arguments{
\item{type}{built in database type accepted values are 1,2,3 and 4. Currently there are 4 different builtin database;
Type 1
Requires ~ 5 GB free storage
Included datasets hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro
Included uniprot proteins and ensembl genomes belongs to following organisms

homo_sapiens 9606 --> ensembl
danio_rerio 7955 zebrafish --> ensembl
gallus_gallus 9031 chicken --> ensembl
mus_musculus 10090 --> ensembl
Rattus norvegicus 10116 ---> ensembl
saccharomyces_cerevisiae 4932--> ensembl,ensembl_fungi
arabidopsis_thaliana 3702--> ensembl_plants
drosophila_melanogaster 7227 --> ensembl,ensembl_metazoa
caenorhabditis_elegans 6239 --> ensembl,ensembl_metazoa
Escherichia coli 562 --> ensembl_bacteria
Escherichia coli str. K-12 substr. MG1655 511145 --> ensembl_bacteria
Escherichia coli K-12 83333 --> ensembl_bacteria

Type 2
Requires ~ 5 GB free storage
Instead of genomes in in the type 1 it contains human and all the mouse strains genomes with their uniprot proteins.
In addition hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro datasets are included

Type 3
Requires ~ 4 GB storage
Contains no genome but it contains all the uniprot data with hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro

Type 4
Requires ~ 13 GB storage
Contains no genome but full uniprot and chembl data with hgnc,hmdb,taxonomy,go,efo,eco,chebi,interpro}
}
\value{
returns empty
}
\description{
Pre build biobtree database for commonly studied datasets and model organism genomes. Once this function called it retrieves
the pre build database saves to users output directory.
}
\examples{

bbUseOutDir(tempdir()) # temp dir for demo purpose
bbBuiltInDB("demo") # small demo database for real database use 1, 2, 3 or 4


}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\docType{class}
\name{bbConfig-class}
\alias{bbConfig-class}
\title{Class for biobtreeR config}
\description{
This class holds the datasets meta data and web service endpoints and used while executing the search/mapping queries.
Instance of this class with name bbConfig is globally set by bbStart function. About dataset meta data, this class instance
holds all the datasets unqiue identifers, entry url templates etc. In biobtree each dataset has unique character
and numeric identifier. For instance Uniprot's charachter identifier is "uniprot" and numeric identifier is 1.
When performing queries the dataset charachter identifier is used for convinience but in actual database it is saved numerically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mapping.R
\name{bbMapping}
\alias{bbMapping}
\title{Chain mapping and filtering}
\usage{
bbMapping(terms, mapfilter, page = NULL, source = NULL, lite = TRUE,
  limit = 1000, inattrs = NULL, attrs = NULL,
  showInputColumn = FALSE)
}
\arguments{
\item{terms}{Input terms for the mapping. Same with search functionality they can be comma seperated identifers or keywords}

\item{mapfilter}{Mapping query which consist of map and optional filter functions in the form of map(dataset).filter(Boolean query expression)
The boolean expressions are based on datasets attributes and dataset attributes can be list with bbAttr function.
Dataset attributes which used in the filters starts with their dataset name. In biobtree boolean expressions
feature has been implemented via Google common expression language so its full capability can be checked in its documentation.}

\item{page}{Optional parameter works similar with bbSearch page paramter.}

\item{source}{Optional dataset identifiers for searching input terms within the given dataset.}

\item{lite}{By default it is TRUE and allow function return quickly with data.frame with mapping identifiers and attributes. If set to TRUE function return raw results converted from json.}

\item{limit}{Limits the number of mapping results. By default without any limit all the results returned.}

\item{inattrs}{Optional comma seperated attribute names for input identifiers and if available their values includes in result data.frame}

\item{attrs}{Optional comma seperated attribute names for mapping identifiers and if available their values includes in result data.frame}

\item{showInputColumn}{Optional logical parameter to show the input identifers in the result data.frame}
}
\value{
returns mapping results in data.frame by default if lite set it true returns json object
}
\description{
Chain mapping identifiers or keywords with filtering and retrieving attributes if available.
}
\examples{
bbStart()

bbMapping("tpi1",'map(uniprot)')

bbMapping("shh",'map(ensembl)')

\dontrun{
# run these examples with building the default dataset with bbBuildData()
#Map protein to its go terms and retrieve go term types
bbMapping("AT5G3_HUMAN",'map(go)',attrs = "type")

#Map protein to its go terms with filter by its type and retrieve their types
bbMapping("AT5G3_HUMAN",'map(go).filter(go.type=="biological_process")',attrs = "type")

#Map gene names to exon identifiers and retrieve the region
bbMapping("ATP5MC3,TP53",'map(transcript).map(exon)',attrs = "seq_region_name")

#Map Affymetrix identifiers to Ensembl identifiers and gene names
bbMapping("202763_at,213596_at,209310_s_at",source ="affy_hg_u133_plus_2"
,'map(transcript).map(ensembl)',attrs = "name")

}

}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server.R
\name{bbStop}
\alias{bbStop}
\title{Stop biobtree}
\usage{
bbStop()
}
\value{
returns empty
}
\description{
Stops running background biobtree process which started with \code{bbStart}
}
\examples{
bbStop()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entry.R
\name{bbEntry}
\alias{bbEntry}
\title{Retrieve entry}
\usage{
bbEntry(identifer, source)
}
\arguments{
\item{identifer}{Identifer for the entry. Note that keywords are not accepted. For instance insted of "vav_human" keyword "p15498" identifier must be passed}

\item{source}{Dataset identifier}
}
\value{
returns biobtree json object
}
\description{
Returns entry for an identifier and dataset. Entry contains all the data raw data for and entry such as mappings, attiributes and paging info if exists.
}
\examples{

bbStart() # if not already started
bbEntry("HGNC:12009","hgnc")

}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entry.R
\name{bbEntryFilter}
\alias{bbEntryFilter}
\title{Retrieve entry with filtered dataset}
\usage{
bbEntryFilter(identifer, source, filters, page = NULL)
}
\arguments{
\item{identifer}{Identifer for the entry.}

\item{source}{Dataset identifier}

\item{filters}{Comma seperated dataset identifer to retrieve}

\item{page}{Page index if results is more than default biobtree paging size.}
}
\value{
returns biobtree json object
}
\description{
Similar with entry retrieval but filtered mapping entries with given datasets.
}
\examples{


bbStart() # if not already started
bbEntryFilter("HGNC:12009","hgnc","uniprot,ensembl")


}
\author{
Tamer Gur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildData.R
\name{bbBuildCustomDB}
\alias{bbBuildCustomDB}
\title{Build custom DB}
\usage{
bbBuildCustomDB(taxonomyIDs = NULL, rawArgs = NULL)
}
\arguments{
\item{taxonomyIDs}{Comma seperated list of taxonomy identifiers for building the genomes}

\item{rawArgs}{For using all available biobtree command line arguments directly}
}
\value{
returns empty
}
\description{
biobtree covers all the genomes in ensembl and ensembl genomes. If the the studied organism genome is not included
in the default pre built in databases then this function is used and build the biobtree database locally for given genomes.
}
\examples{

\dontrun{

bbUseOutDir("your directory path")
bbBuildCustomDB(taxonomyIDs="1408103,206403")

}

}
\author{
Tamer Gur
}
